%% Sym-LSM medium demo
%
% The demo routine demo_LSM_medium.m shows how the noise variance 
% estimation is integrated into the matching process. Again the required 
% input data are loaded from file. Here these are the two complete images 
% together with two corresponding Lowe-keypoints (coordinates, scale, 
% direction). These are used to define the window size and the approximate 
% values. The noise variance functions vg and vh are determined 
% automatically from the area around the keypoints. First, the input images 
% with the keypoints are shown. When zooming into the keypoints the centre 
% and the direction vector fixing the scale and the direction can be seen. 
% Further figures show the selected windows, the noise standard deviations 
% and the change of the estimated image windows.
%
% wf 7/2020

close all
clearvars
%clc

addpath(genpath('src/')) 

disp(' --------------------------------------------------------------------- ')
disp(' ---- Symmetric LSM with 6 geometric and 2 radiometric parameters ---- ')
disp(' -------------- two images with corresponding keypoints -------------- ')
disp(' --------------------------------------------------------------------- ')



%% set control parameters
plot_type       = 1;        % medium output
max_iter        = 9;        % maximum number of iterations
sigma_smooth    = 0;        % smoothing of estimated reference image

%% provide input data
% --- load data ---------------------------------------------------------
load('example_data/Sym_LSM_medium_demo');
% Image_l, Image_r       images
% X                      2x4 matrix of corresponding keypoints
%                        [x,y,scale,direction]

figure('name','left and right images')
subplot(1,2,1)
imshow(Image_l)
    hold on
    plot_circle_direction_with_background(X(1,2),X(1,1),3*X(1,3),X(1,4),8,4)
title('left')
subplot(1,2,2)
imshow(Image_r)
    hold on
    plot_circle_direction_with_background(X(2,2),X(2,1),3*X(2,3),X(2,4),8,4)
title('right')


%% select windows ======================================================
% half size of window in left image: 4 * Lowe-scale
Nig = round(4*X(1,3));
Nih = round(4*X(2,3));

% windows
g = round(mean(Image_l(X(1,1)-Nig:X(1,1)+Nig,X(1,2)-Nig:X(1,2)+Nig,:),3));
h = round(mean(Image_r(X(2,1)-Nih:X(2,1)+Nih,X(2,2)-Nih:X(2,2)+Nih,:),3));

figure('name','left and right image window')
subplot(1,2,1)
imshow(g/255)
title('left')
subplot(1,2,2)
imshow(h/255)
title('right')


%% determine noise standard deviations of surrounding area
DD = 100; % 10000 pixels should be sufficient

% --- estimate noise standard deviations of Image_l ---------------------
% determine bounding box within Image_l
[M,N] = size(Image_l);
x_min = max([1,X(1,1)-DD]); y_min = max([1,X(1,2)-DD]);
x_max = min([M,X(1,1)+DD]); y_max = min([N,X(1,2)+DD]);
% estimate noise standard deviations
[s_g,m_g,~] = noise_standard_deviation_estimation(Image_l(x_min:x_max,y_min:y_max,:));

% --- estimate noise standard deviations of Image_l ---------------------
% determine bounding box within Image_r
[M,N] = size(Image_r);
x_min = max([1,X(2,1)-DD]); y_min = max([1,X(2,2)-DD]);
x_max = min([M,X(2,1)+DD]); y_max = min([N,X(2,2)+DD]);
% estimate noise standard deviations 
[s_h,m_h,~] = noise_standard_deviation_estimation(Image_r(x_min:x_max,y_min:y_max,:));

% variances in pixels^2
vg      = (mean(s_g,1)'*256).^2;
vh      = (mean(s_h,1)'*256).^2;
% --- show noise standard deviations vg and vh (256x1) -----------------
figure('name','noise standard deviations');
hold on
plot(mean(s_g,1)*255,'-b','LineWidth',2);
plot(mean(s_h,1)*255,'-r','LineWidth',2);
title('Noise standard deviations: left (blue), right(red)')
xlim([min([m_g(:);m_h(:)]),max([m_g(:);m_h(:)])]*255)
ylim([0,1.1*max([mean(s_g,1)'*255;mean(s_h,1)'*255])])
   
%% === approximate values ==============================================
% --- approximate affinity referring to centre of window ---------------
D       = [X(2,3)/X(1,3)*eye(2),[0;0];0 0 1]; % relative scaling
alpha   = X(2,4)-X(1,4);                      % rotation
R       = [cos(alpha) -sin(alpha) 0;sin(alpha) cos(alpha) 0;0 0 1];
A_a     = D*R;                                % similarity
% --- approximate radiometric transformation --------------------------
R_a     = [1,0];
% minimum size of overlap
Nf_MIN = 4;

% Approximate geometric affinity z = B y
disp([ 'approximate scale difference                                 : ',num2str(X(2,3)/X(1,3))])
disp([ 'approximate rotation difference                              : ',num2str(alpha*180/pi), ''])
disp( 'approximate geometric affinity       z = A y                 : ')
disp(A_a);

disp(['approximate intensity transformation h = b7*y + b8 [gr] : [',...
    num2str(R_a(1)),',',num2str(R_a(2)*255),']'])
x_approx = [reshape(A_a(1:2,:),6,1);R_a'];

% ---  maximum number of iterations N_iter ------------------------------
disp(['Maximum number of iterations                             : ', num2str(max_iter)])

% ---  minimum size of overlap  ------------------------------
disp(['Minimum size of overlap                                  : ', num2str(Nf_MIN),'x', num2str(Nf_MIN)])

% ---  smoothing kernel sigma_smooth ------------------------------------
disp(['Smoothing kernel [pixels]                                : 0'])

 
%% === main routine Sym-LSM =============================================

disp('=== Iteration sequence =======================================')

[est_x,est_sigma_0,NN,Nf,Red,N_iter] = LSM_62_sym_warp_main...
    (g,h,vg,vh,A_a,R_a,sigma_smooth,max_iter,Nf_MIN,1);

%% === show results =====================================================
A_est   = [reshape(est_x.x(1:6),2,3);0 0 1];
    Rm_est  = [ est_x.x(7:8)';0 1];
    Ai_est  = A_est;
    
image_pair_analyse(...
    A_a,  ...
    R_a, ...
    Ai_est, ...
    Red, ...
    est_sigma_0, ...
    est_x, ...
    1,0);

disp(' --------------------------------- ')
disp(' ----  end demo symmetric LSM ---- ')
disp(' --------------------------------- ')


