%% Check correctness of Sym-LSM
%
% The demo routine	demo_LSM_simulated.m is meant to check the 
% correctness of the implementation based on simulated data. It 
% allows to monitor the individual iterations for a single case, 
% or to statistically test, whether the resultant parameters and 
% their covariance matrix are coherent with the theoretical values, 
% the true values of the parameters and the theore2018tical covariance 
% matrix derived by the estimation procedure (Cramer-Rao bound).
%
% symmetric model
% ===============
% g(z) = 1/a_7 * (f(B  * z + a ) - a_8) + m_h(z)
% h(z) =   a_7 *  f(Bi * z + ai) + a_8  + n_h(z)
% B=[a1 a3 a5]   S = [a7 a8]
%   [a2 a4 a6]       [0  1 ]
%   [0  0  1 ]
% A = B^2        R = S^2
%
% main parameters
% ===============
% img_filename  = name of image file for generating true image
%                 'corner': corner image, simulated
%                 'filename': read from file
% s,t           = parameters for noise sigma_n^2=s+t*f
% Tg            = geometric parameters 2x3 matrix
%                  [a1 a3 a5]
%                  [a2 a4 a6]
% Tr            = radiometric parameters [a_7,a_8]'
% Ng            = size of first image
% Nh            = size of second image
% sigma         = smoothing kernel size, if = 0: no smoothing
% init_rand     = number of random generator, 0 --> arbitrary
% N_samples     = number of samples generated
%                 (for checking covariance matrix)
% N_iter        = number of iterations
%
% wf 11/2018, 01/2020, 7/2020

clearvars
close all
%clc

disp(' --------------------------------------------------------------------- ')
disp(' ---- symmetric LSM with 6 geometric and 2 radiometric parameters ---- ')
disp(' --------------------------------------------------------------------- ')

addpath(genpath('src/'))


%% === Choose your own parameters =======================================
% --- initial random number (default = 0) -------------------------------
par.init_rand = 2;
% init_rand = 0;

% --- Number of samples (default = 50) ----------------------------------
par.N_samples = 50;                  
% par.N_samples = 1; 

% --- Type of data, in [0..4], default = 4 ------------------------------
% par.type_data = 1;  % 1   square
% par.type_data = 2;  % 2   waves
% par.type_data = 3;  % 3   MIT-image, subwindow
 par.type_data = 4;   % 4   smoothed noise
 
% --- test symmetry, only has an effect if N_samples=1 -----------------
par.test_symmetry=0;
%par.test_symmetry=1;

% --- use warping routine ---
par.warping_routine =1 ;
% par.warping_routine =0 ;

%% === Set and display main parameters ==================================
par = simulated_set_parameters(par);


simulated_display_parameters(par);

par = simulated_prepare_true_values(par);

%% generate reference image ============================================

[I_true,f_true,g_true,h_true,var] = ...
    generate_reference_image(par);

%% start samples, may be only one ======================================

% set statistics
sigma_0s     = zeros(par.N_samples,1);
est_xs       = zeros(par.N_samples,8);
N_iters      = zeros(par.N_samples,1);
NNs          = zeros(par.N_samples,1);
init_randoms = zeros(par.N_samples,1);

% for measuring CPU time
lsm_time=0;
r_sample = 0;
for sample = 1:par.N_samples
    
    monitor_samples(sample,par.N_samples);
    
    % --- Generate f_true, g, and h from I_true ------------------------
    st = randn('state');
    par.init_randoms(sample)= st(2);
    
    [g,h,std_i_g,std_i_h] = LSM_62_sym_generate_noisy_observing_images...
        (g_true,h_true,par);
    
    if sample == 1
        figure('name','first sample image pair')
        subplot(1,2,1)
        imshow(g/255);
        title('noisy left image')
        subplot(1,2,2)
        imshow(h/255);
        title('noisy right image')
    end
    
    % --- adapt noise: take smoothing of signal into account -----------
    vgm=par.vg + 1/sqrt(par.R_a(1)) * std_i_g^2*0.38;
    vhm=par.vh +   sqrt(par.R_a(1)) * std_i_h^2*0.38;
    
    
    % ---- estimate transformation (LSM_62_main) -----------------------
    % set parameters
    A_a             = par.A_a;          % 3x3 matrix, approx. geom. transf.
    R_a             = par.R_a;          % 1x2 vector, approx. rad. transf.
    sigma_smooth    = par.sigma_smooth; % smoothing parameter
    max_iter        = par.max_iter;     % maximal number of iterations
    Nf_MIN          = par.Nf_MIN;       % minimum size of overlap 
    pt              = par.plot_type;    % output level
    
    tic;
    
    if par.warping_routine
        [est_x,est_sigma_0,NN,Nf,Red,N_iter] = ...
            LSM_62_sym_warp_main(g,h,vgm,vhm,A_a,R_a,sigma_smooth,max_iter,Nf_MIN,pt);
    else
        [est_x,est_sigma_0,NN,Nf,Red,N_iter] = ...
            LSM_62_sym_main(g,h,vgm,vhm,A_a,R_a,sigma_smooth,max_iter,Nf_MIN,pt);
    end
    
    
    est_xi = est_x;
    if Red > 0 &&  N_iter < par.max_iter 
        % successful matching
        r_sample = r_sample+1;
        
        lsm_time=lsm_time+toc;
        
        sigma_0s(r_sample)   = est_sigma_0;
        est_xs(r_sample,:)   = est_x.x';
        N_iters(r_sample)    = N_iter;
        NNs(r_sample)        = NN;
         
        % If N_samples = 1 and test_symmetry = 1
        if par.N_samples == 1 &&  par.test_symmetry
            disp('--- start symmetric estimation h->g ---')
            % set parameters
            Ai_a = inv(par.A_a);
            Ri_a = [1/par.R_a(1),-par.R_a(2)/par.R_a(1)];
            % estimate
            [est_xi,est_sigma_0i,NN,Nf,Redi,N_iter] = ...
                LSM_62_sym_warp_main...
                (h,g,vhm,vgm,Ai_a,Ri_a,sigma_smooth,max_iter,Nf_MIN,pt);
            
            if par.plot_type > 1
                Ai_est  = [reshape(est_xi.x(1:6),2,3);0 0 1]
                Ri_est  = [est_xi.x(7:8)';0 1]
            end
        end
    end
    
end


%% analyse 

par.plot_type = 1;

sym_lsm_analysis(...
    r_sample,...
    est_sigma_0,...
    est_x,...
    est_xi,...
    est_xs, sigma_0s, N_iters,...
    Red,...
    lsm_time,...
    par);


disp(' --------------------------------- ')
disp(' ----  end demo symmetric LSM ---- ')
disp(' --------------------------------- ')


% file_name = ['LSM_sym_par_pyr_result_',num2str(init_rand_prolog),'_',num2str(init_rand_samples),'_',num2str(type_data,'%02.f'),'.mat']
% save(file_name)


return