% LSM 62 symmetric: estimate parameters
%
% function [est_x,d_x,est_sigma_0,NN,Red,Xm,v]  = ...
%    LSM_62_sym_estimate_parameters(fe,fes,g,h,yi,zi,w,xa,pt)
%
% fe          = NfxNf estimated image
% fes         = NfxNf smoothed image
% g,h         = NgxNg, NhxNh observations of image
% yi,zi       = Kg, Kh list of observed pixels
% w           = Kg+Kh vector of weights
% xa          = actual estimates for geometric and radiometric transformation
% pt          boolean for plot/print
%
% est_x       = updated estimate (.x = 8-vector, .C = 8x8 CovM
% d_x         = last correction of est_x.x
% est_sigma_0 = sqrt ofsqrt of  estimated variance factor
% NN          = number of observing pixels
% Red         = redundancy
% Xm          = design matrix
% v           = residuals
%
% wf 11/2018 

function [est_x,d_x,est_sigma_0,NN,Red,Xm,v]  = ...
    LSM_62_sym_estimate_parameters(fe,fes,g,h,yi,zi,w,xa,pt)

%% affinities
B    = [xa(1) xa(3) xa(5);xa(2) xa(4) xa(6);0 0 1];
Bi   = inv(B);
S    = [xa(7);xa(8)];
fak  = S(1);
faki = 1/fak;

%% half size of images
Nf = (size(fe,1)-1)/2;
Ng = (size(g,1)-1)/2;
Nh = (size(h,1)-1)/2;

%% number of observations
Kg   = size(yi,2);
Kh   = size(zi,2);

%
NN  = Kg+Kh;
NU  = 8;
% N - (8 geometric parameters + interior of fe)
% interior = round(sqrt(Kg*Kh))
Red = NN-(NU+round(sqrt(Kg*Kh)));

%
dg = zeros(NN,1);
Xm = zeros(NN,8);

%% first image
for k = 1:Kg
    i=yi(1,k)-Ng;                   % coordinates in g (centred)
    j=yi(2,k)-Ng;
    xh = B*[i;j;1];                 % real coordinates in f (centred)
    xc=xh(1);
    yc=xh(2);
    x=xc+Nf+1;                      % pixel coordinates in f
    y=yc+Nf+1;
    %
    fij  = LSM_f_cubic_interpolation(x,y,fe);    % estimated signal
    fxij = LSM_fx_cubic_interpolation(x,y,fes);  % gradient at signal
    fyij = LSM_fy_cubic_interpolation(x,y,fes);  % consistent with fij
    %     fxij = LSM_fx_cubic_smoothing(x,y,fes);    % gradient at smoothed signal
    %     fyij = LSM_fy_cubic_smoothing(x,y,fes);    % see parallel implementation
    dg(k)  = g(i+Ng+1,j+Ng+1)/255 - (fij-S(2))/S(1);    % g: 255->1
    % Jacobian
    Xm(k,:) = [fxij*i fyij*i fxij*j fyij*j fxij fyij -(fij-S(2))*faki -1]*faki;
    
    % Check Jacobian numerically, are ok
    % [my,Syy,J]=var_prop_classical(@f,mx,Sxx,p)
    %     if k < 10
    %         [fij_c,~,Jg] = var_prop_classical(@gj,xa,0.0000000001*ones(8),[i;j;fe(:)])
    %         (fij-S(2))/S(1)
    %         Xm(k,:)
    %         [fij_c-(fij-S(2))/S(1),Jg-Xm(k,:)]
    %     end
end

%% second image
for k=1:Kh;
    i=zi(1,k)-Nh;                   % pixel coordinates in h (centred)
    j=zi(2,k)-Nh;
    xh = Bi*[i;j;1];                % real coordinates in f (centred)
    xc=xh(1);
    yc=xh(2);
    x=xc+Nf+1;                      % pixel coordinates in f
    y=yc+Nf+1;
    %
    fij  = LSM_f_cubic_interpolation(x,y,fe);    % estimated signal
    fxij = LSM_fx_cubic_interpolation(x,y,fes);  % gradient at signal
    fyij = LSM_fy_cubic_interpolation(x,y,fes);  % consistent with fij
    %     fxij = LSM_fx_cubic_smoothing(x,y,fes);  % gradient at smoothed signal
    %     fyij = LSM_fy_cubic_smoothing(x,y,fes);  % see parallel implementation
    pij  = Bi(1:2,1:2)'*[fxij;fyij]; % gradient in f
    pxij = pij(1);
    pyij = pij(2);
    % residuals
    dg(Kg+k) = h(i+Nh+1,j+Nh+1)/255 - (S(1)*fij+S(2));     % h:255->1
    % Jacobians
    Xm(Kg+k,:) = -[pxij*xc pyij*xc pxij*yc pyij*yc pxij pyij -fij*faki -faki]*fak;
    
    % Check Jacobian numerically, are ok
    % [my,Syy,J]=var_prop_classical(@f,mx,Sxx,p)
    %     if k < 10
    %         [fij_c,~,Jh] = var_prop_classical(@hk,xa,0.0000000001*ones(8),[i;j;fe(:)])
    %         (S(1)*fij+S(2))
    %         Xm(Kg+k,:)
    %         [fij_c-(S(1)*fij+S(2)),Jh-Xm(Kg+k,:)]
    %     end
end


%% Normal equation system
N  = Xm' * (Xm   .* repmat(w,1,NU));
n  = Xm' * (dg .* w);

%% Inverse and solution
C_xx     = inv(N);
d_x      = C_xx * n;

est_x.x  = xa + d_x;
est_x.C  = C_xx / 255^2;       % since weights refer to [0..255]

%% residuals
v     = Xm * d_x - dg;
Omega = v' * (v.*w) * 255^2;   % since weights refer to [0..255]
est_sigma_0 = sqrt(Omega/Red);


%% plot/print

if pt > 1
    N           = N;
    n           = n';
    dx          = d_x'
    est_x_      = est_x.x'
    sigma_0_est = est_sigma_0
    Covariance_2_sigma_rho(est_x.C);
    sigmas      = diag(sqrt(est_x.C))';
    B_est       = reshape(est_x.x(1:6),2,3)
    Sigmas_B    = reshape(sigmas(1:6),2,3)
    S_est       = est_x.x(7:8)'
    Sigmas_S    = sigmas(7:8)
    
    figure('Name','Histograms of residuals')
    subplot(1,2,1)
    hist(v(1:Kg),ceil(sqrt(Kg)))
    title({['v\_g: mean = ',num2str(mean(v(1:Kg)))],...
        ['v\_g: std  = ',num2str(std(v(1:Kg)))]})
    subplot(1,2,2)
    hist(v(Kg+1:Kg+Kh),ceil(sqrt(Kh)))
    title({['v\_h: mean = ',num2str(mean(v(Kg+1:Kg+Kh)))],...
        ['v\_h: std  = ',num2str(std(v(Kg+1:Kg+Kh)))]})
end

return