% LSM_find_observation_positions in the common area
% with distance to boundary < BOUND
%
% [Nf,yi,zi,w] =
% LSM_62_sym_find_observation_positions(g,h,vg,vh,xa,BOUND,pt)
% 
% g1, g2 = two images
% vg,vh  = parameters (s,t) variance functions, e.g. V(g)=vg(1)+vg(2)*g
% xa     = parameters of approximate transformations
% Nf_MIN = minimum size of overlap (radius of window f)
% pt     = boolean for plot/print
%
% Nf     = size of refence image
% yi     = lists of (i,j) coordinates in g
% zi     = lists of (i,j) coordinates in h
% w      = vector of weights
%
% wf 08/2012

function [Nf,yi,zi,w]= ...
    LSM_62_sym_find_observation_positions(g,h,vg,vh,xa,Nf_MIN,pt)
%% Constant, control
BOUND  = 3;

%% size of images
Ng=(size(g,1)-1)/2;
Nh=(size(h,1)-1)/2;

%% determine transfromation y->x, x->h
B  = [xa(1) xa(3) xa(5);xa(2) xa(4) xa(6);0 0 1];
Bi = inv(B);

%% find region in x-space
% transform boundary, take 3-pixel distance
Ngr   = Ng-BOUND;
bbg   = square_corners(Ngr);    % bb in g
bbgf  = B*bbg;                  % corresponding quadrangle in f
bbgfp = [bbgf,bbgf(:,1)];       % closed

Nhr   = Nh-BOUND;
bbh   = square_corners(Nhr);    % bb in h
bbhf  = Bi*bbh;                 % corresponding quadrangle in f
bbhfp = [bbhf,bbhf(:,1)];       % closed
% find size of square, check 4 diagonals
dmin=10000;
for i=1:4
    % direction of i-th diagonal
    u   = cos(pi/4*(2*i-1));
    v   = sin(pi/4*(2*i-1));
    % for each side
    for j=1:4
        % distances to polygon g and h
        distg = dist_poly_ray(bbgfp,j,u,v);
        disth = dist_poly_ray(bbhfp,j,u,v);
        % take minimum
        dmin  = min([dmin,distg,disth]);
    end
end
% sizeof square of f
Nf  = floor((dmin)/sqrt(2))-1;   %% -1 appears to be necessary
Nfr = Nf-BOUND;

if Nf < Nf_MIN
    yi=0;
    zi=0;
    w=0;
    return
end


%% pixels in the first image g
k=0;
yi=[];
yi=[];
fg=[];
w=[];
for i=-Ngr:Ngr
    for j=-Ngr:Ngr
        xh = B*[i;j;1];  % coordinates from g transformed into f
        x  = xh(1)/xh(3);
        y  = xh(2)/xh(3);
        % check offset from boundary in second image
        if (round(x) >  -Nfr) && (round(x) <  Nfr) ...
                && (round(y) >  -Nfr) && (round(y) <  Nfr)
            k=k+1;                              % next pixel in g
            % store coordinates
            % referring to [-Ng,-Ng] (pixels are +1)
            yi(1:2,k)=[i+Ng;j+Ng];
            w(k) = 1/vg(g(i+Ng,j+Ng)+1);   % determine weight
        end
    end
end
% number of pixels in g
Kg=size(yi,2);

%% pixels in the second image z, away by BOUND from the boundary
k=0;
zi=[];
zi=[];
fh=[];
for i=-Nhr:Nhr
    for j=-Nhr:Nhr
        xh  = Bi*[i;j;1];  % coordinates from h transformed into f
        x1  = xh(1)/xh(3);
        y1  = xh(2)/xh(3);
        % check offset from boundary in first image
        %if (x1 >= 1+BOUND) && (x1 <= N1-BOUND) && (y1 >= 1+BOUND) && (y1 <= N1-BOUND)
        if (round(x1) > -Nfr) && (round(x1) < Nfr) ...
                && (round(y1) > -Nfr) && (round(y1) < Nfr)
            k=k+1;                                  % next pixel in g
            % store coordinates
            % referring to [-Nh,-Nh] (pixels are +1)
            zi(1:2,k)=[i+Nh;j+Nh];
            w(Kg+k) = 1/vh(h(i+Nh,j+Nh)+1);    % determine weight
        end
    end
end
% number of pixels in h
Kh=size(zi,2);

% enforce column of w
w = w(:);

if pt > 1
    
    figure('Name','areas in middle image')
    hold on
    axis equal
    Nfb=Nf-BOUND;
    % plot bb for f
    
    plot_quadrangle([-Nf,-Nf,Nf,Nf;...
        -Nf,Nf,+Nf,-Nf],'-r');
    % plot bb reduced for f
    plot_quadrangle([-Nfb,-Nfb,Nfb,Nfb;...
        -Nfb,Nfb,+Nfb,-Nfb],'--r')
    plot_quadrangle(bbgfp,'-g')
    plot_quadrangle(bbhfp,'-b')
    plot(1.5*[-Nf,Nf],1.5*[+Nf,-Nf],'--k')
    plot(1.5*[-Nf,Nf],1.5*[-Nf,+Nf],'--k')
    
    % second image  in coordinate system f
    xih = Bi*[zi-Nh*ones(2,Kh);ones(1,Kh)];
    plot(xih(1,:),xih(2,:),'og','MarkerSize',8)
    
    % first image in coordinate system f
    xig = B*[yi-Ng*ones(2,Kg);ones(1,Kg)];
    plot(xig(1,:),xig(2,:),'ob','MarkerSize',8)
    
end
return