%% LSM_62_sym_generate_true_observing_images
%
% generating true images for symmetric LSM
% call
% [f_true,g_true,h_true] = LSM_62_sym_generate_true_observing_images...
%     (Ni,I_true,par,plot_type)
%
% I_true            true image
% plot_type         in [0,1,2] controlling output
%
% f_true            true reference image
% g_true,h_true     true observing images
% std_g, std_h      standard deviations of bi-cubic interpolation error
%
% wf 7/2020

function [f_true,g_true,h_true,par] = ...
    LSM_62_sym_generate_true_observing_images(I_true,par)

%% generate f_true, observing g, and h

a7_true = par.S_true(1);
a8_true = par.S_true(2);
Bi_true = inv(par.B_true);
% square f
Nf    = par.Nfr+par.BOUND;
bbf   = square_corners(Nf);

Ni = (size(I_true,1)-1)/2;

%% generate true image f including boundary
Nf=par.Nfr+par.BOUND;
f_true = zeros(2*Nf+1);
for i=-Nf:Nf
    for j=-Nf:Nf
        x  = i+Ni+1;
        y  = j+Ni+1;
        % true signal basis: real valued
        f_true(i+Nf+1,j+Nf+1) = I_true(x,y);
    end
end

%% generate first image
% size
bbfg   = Bi_true*bbf;
Ngr    = ceil(max(abs(bbfg(:))));
Ng     = Ngr+par.BOUND;
g_true = zeros(2*Ng+1);
g      = zeros(2*Ng+1);
% fill
for i=-Ng:Ng
    for j=-Ng:Ng
        xh = par.B_true * [i;j;1];
        x  = xh(1)+Ni+1;
        y  = xh(2)+Ni+1;
        % true signal basis: real valued
        g_true(i+Ng+1,j+Ng+1) = ...
            (LSM_f_cubic_smoothing(x,y,I_true)-a8_true*255)/a7_true;
    end
end

%% generate second image
% size
bbfh  = par.B_true*bbf;
Nhr   = ceil(max(abs(bbfh(:))));
Nh    = Nhr+par.BOUND;
h_true=zeros(2*Nh+1);
h     =zeros(2*Nh+1);
for i=-Nh:Nh
    for j=-Nh:Nh
        xh = Bi_true * [i;j;1];
        x  = xh(1)+Ni+1;    % coordinates referring to corner
        y  = xh(2)+Ni+1;
        % true signal basis: real valued
        h_true(i+Nh+1,j+Nh+1) = ...
            (a7_true*(LSM_f_cubic_smoothing(x,y,I_true))+a8_true*255);
    end
end

diff_g = conv2(g_true,par.Diff_filter,'valid');
diff_h = conv2(h_true,par.Diff_filter,'valid');
par.std_g  = std(diff_g(:));
par.std_h  = std(diff_h(:));

%% plots

if par.plot_type > 0
    figure('Name','mean, left, right: generated true images')
    subplot(2,3,2);imshow(f_true/255);
    title(['f-true at [c,r] = [',num2str(par.col),',',num2str(par.row),'] (',...
        num2str(size(f_true,1)),'x',num2str(size(f_true,2)),')']);
    axis equal;
    subplot(2,3,4);imshow(g_true/255);
    title(['g-true (',num2str(size(g_true,1)),'x',num2str(size(g_true,2)),')']);axis equal;
    subplot(2,3,6);imshow(h_true/255);
    title(['h-true (',num2str(size(h_true,1)),'x',num2str(size(h_true,2)),')']);axis equal;
    colormap(gray)
    
end
end

