function par = simulated_set_parameters(par)
%% sets all control parameters
%
% wf 7/2020
%
% par.BOUND             boundary, constant
% par.init_rand         initial random number
% par.type_data         type of image generation
% par.row, par.col      row and column of window in image
% par.bandwidth         1/bandwidth = average wavelength
% par.mean_f            of image
% par.sigma_f           of image
% par.K                 number of generated frequencies
% par.dx                pixel distance (do not change)
% par.sigma_s           maximum change of geometric affinity
% par.sigma_r           maximum change of radiometric affinity
% par.test_symmetry     0/1: test on symmetry
% par.warping_routine   0/1: use matplab warping routine imtransform.m
% par.N_samples         number of samples
% par.plot_type         in [0,1,2]: contorl of output
% par.Nfr               half window size of indented overlap
% par.A_true            3x2, true geometric transformation
% par.A_a               3x2, approximate geometric transformation
% par.R_true            1x2, true geometric transformation
% par.R_a               1x2, approximate geometric transformation
% par.max_iter          maximum number of iterations
% par.Nf_MIN            minimum size of overlap (radius of f)
% par.S                 significance level
% par.sigma_n           noise standard deviation
% par.img_filename      image file name stored intermediately
% par.stg               parameters for variance function v=s+t*g
% par.sth               parameters for variance function v=s+t*h
% par.sigma_smooth      smoothing parameter for f
% par.Diff_filter       kernel for bi-cubic interpolation error, constant
%

par.BOUND     = 3;

% --- initial random number --------------------------------------------
if par.init_rand <= 0
    par.init_rand = 0
end
par.init_rand_prolog     = par.init_rand;
par.init_rand_samples    = par.init_rand;
par.init_rand_prolog     = init_rand_seed(par.init_rand_prolog);


% --- type data --------------------------------------------------------

if par.type_data < 1
    % par.type_data = 1;  % 1   square
    % par.type_data = 2;  % 2   waves
    % par.type_data = 3;  % 3   MIT-image, subwindow
    par.type_data = 4;    % 4   smoothed noise
end

par.row=0;par.col=0;
switch par.type_data
    case 1 % for square
        par.par  = [3,0.7];   % size 1/par(1), smoothness of border
        par.img_filename = 'square'; % artificial image
    case 2 % for waves
        par.par  = [6,1.5];  % ~number of waves, smoothness of borders
        par.img_filename = 'waves2'; % artificial image
    case 3 % for position in MIT
        %par.col = 182; par.row = 166;
        par.col = 114; par.row = 65;
        %par.col = 160; par.row = 104;
        %par.col = 193; par.row = 207;
        par.img_filename = 'MIT.png';
        % row and colum of centre of window (only needed if file is read in
    case 4  % degree of smoothing ...
        par.bandwidth =    0.12;     % 1/bandwidth = average wavelength
        par.mean_f    =    128;     % of image
        par.sigma_f   =     25;     % of image
        par.K         =     50;     % number of generated frequencies
        par.dx        =      1;     % pixel distance (do not change)
        par.sigma_s   =    0.1;     % maximum change of geometric affinity
        par.sigma_r   =      1;     % maximum change of radiometric affinity
        
        par.par(1) = par.bandwidth;
        par.par(2) = par.mean_f;
        par.par(3) = par.sigma_f;
        par.par(4) = par.K;
        par.par(5) = par.dx;
        par.par(6) = 4;
        par.par(7) = par.sigma_s;
        par.par(8) = par.sigma_r/255;
        par.img_filename = 'random'; % artificial image
end

% --- noisy variance [gr^2] -------------------------------------------
par.var_n    =  8;
par.sigma_n  =  sqrt(par.var_n);

% ---  noise variance = s + t*g (for the range [0,255]) ----------------
par.stg          = [par.var_n ,0]';
par.sth          = [par.var_n ,0]';

% --- test symmetry, only has an effect if N_samples=1 -----------------
if par.test_symmetry < 0
    par.test_symmetry=1;
end

% --- use warping routine ---
if par.warping_routine < 0;
   par.warping_routine = 1;
end

% --- number of samples for checking covariance matrix -----------------
if par.N_samples <= 0
    par.N_samples = 50;
end

if par.test_symmetry==1
    par.N_samples = 1;
end

par.plot_type = par.N_samples == 1;

% --- window size ------------------------------------------------------
par.Nfr  = 15;

% -- minimum size of overlap (radius of f) -----------------------------
par.Nf_MIN = 4;

%  ----  Geometric affinity z = A y  (! forward) -----------------------
% par.A_t         = [  2,  0.3,  2;...
%                 0.9,    3,  3];
par.A_t         = [  1.2,  0.05,  0.5;...
    0.1,    0.8,  1];
% par.A_t         = [  1.1,  0.05,   0.5;...
%                     0.1,  0.90,  -0.5];
% par.A_t         = [  1.05,  0.05,  0.5;...
%                      0.1,  0.94, -0.5];
% par.A_t         = [  1.0,  0.00,  0.0;...
%                    00,    1,  0];

% specify true and approximate values
par.A_a  = par.A_t;

par.R_true =  [1.05 10];
par.R_a    = par.R_true;

% adapt to graylevel range [0..1]
par.R_true(2)=par.R_true(2)/255;

% ---  smoothing kernel ------------------------------------------------
par.sigma_smooth        = 0;

% --- maximum number of iterations ------------------------------------
par.max_iter  = 7;

% --- significance level ----------------------------------------------
par.S = 0.999;

% kernel for estimating interpolation error
par.Diff_filter = [ ...
    [  1/65536,   -9/32768,   63/65536,   41/16384,   63/65536,   -9/32768,  1/65536]; ...
    [ -9/32768,   81/16384, -567/32768,  -369/8192, -567/32768,   81/16384, -9/32768]; ...
    [ 63/65536, -567/32768, 3969/65536, 2583/16384, 3969/65536, -567/32768, 63/65536]; ...
    [ 41/16384,  -369/8192, 2583/16384, -2415/4096, 2583/16384,  -369/8192, 41/16384]; ...
    [ 63/65536, -567/32768, 3969/65536, 2583/16384, 3969/65536, -567/32768, 63/65536]; ...
    [ -9/32768,   81/16384, -567/32768,  -369/8192, -567/32768,   81/16384, -9/32768]; ...
    [  1/65536,   -9/32768,   63/65536,   41/16384,   63/65536,   -9/32768,  1/65536]];

end

