function [fe,fes,fgi,fhi] = LSM_62_sym_warp_estimate_f(g,h,vg,vh,Nf,xa,sigma,pt)
%% estimate function f from g and h using xa and smooth.
%
% [fe,fes,fgi,fhi] = LSM_62_sym_warp_estimate_f(g,h,vg,vh,Nf,xa,sigma,pt)
%
% g,h       two images
% vg, vh    variance function
% Nf        half size of reference image
% xa        current geometric and radiometic transformation
% sigma     stdv for smoothing
% pt        in [0,1,2]: controling output
%
% fe,fes    estimated and smoothed image
% fgi,fhi   interpolated and transformed images g and h
%
% wf 2018/11

%% geometric transformation
B  = [xa(1) xa(3) xa(5);xa(2) xa(4) xa(6);0 0 1];
S  = [xa(7);xa(8)];
Bi = inv(B);

%% sizes
Ng = (size(g,1)-1)/2;
Nh = (size(h,1)-1)/2;

Mf = 2*Nf+1;

fe  = zeros(2*Nf+1);
fes = zeros(2*Nf+1);
fgi = zeros(2*Nf+1);
fhi = zeros(2*Nf+1);

%% estimate f


% parallel
% combined transformation f->g
BC = [1 0 Ng+1;0 1 Ng+1; 0 0 1]* Bi*[1 0 -Nf-1;0 1 -Nf-1; 0 0 1];
T = maketform('affine',inv(BC)');
xdata = [1,Mf]; ydata = [1,Mf];
fg_v  = imtransform(g',T,'bicubic','xdata',xdata,'ydata',ydata)';
fg_v  = S(1) * fg_v/255 + S(2);

% enforce [0,1]
fg_v = max(0,fg_v);
fg_v = min(1,fg_v);

% weights
wg_v  = 1./vg(round(fg_v(:,:)*255+1))/S(1)^2;

% combined transformation f->h
BC    = [1 0 Nh+1;0 1 Nh+1; 0 0 1] * B * [1 0 -Nf-1;0 1 -Nf-1; 0 0 1];
T     = maketform('affine',inv(BC)');
xdata = [1,Mf]; ydata = [1,Mf];
fh_v  = imtransform(h',T,'bicubic','xdata',xdata,'ydata',ydata)';
fh_v  = (fh_v/255-S(2))/S(1);

% enforce [0,1]
fh_v = max(0,fh_v);
fh_v = min(1,fh_v);

% weights
wh_v  = 1./vh(round(fh_v(:,:)*255+1))*S(1)^2;

% weighted average
fe_v = (wg_v.*fg_v+wh_v.*fh_v)./(wg_v+wh_v);


% % sequential
% for i=-Nf:Nf
%     for j=-Nf:Nf
%
%         % image g: dermine g-values of grid in f
%         xgij = Bi * [i;j;1];
%         xg = xgij(1) + Ng+1;
%         yg = xgij(2) + Ng+1;
%         if xg > 2*Ng || yg > 2*Ng || xg < 2  || yg < 2
%             keyboard
%         end
%         fg = S(1)*LSM_f_cubic_interpolation(xg,yg,g)/255+S(2);
%
%         % enforce range [0,1]
%         fg=min(fg,1);
%         fg=max(fg,0);
%
%         wg = 1/vg(round(fg*255)+1)/S(1)^2;
%         fgi(i+Nf+1,j+Nf+1) = fg;
%
%         % image h: dermine h-values of grid in f
%
%         xhij = B * [i;j;1];
%         xh = xhij(1) + Nh+1;
%         yh = xhij(2) + Nh+1;
%         if xh > 2*Nh || yh > 2*Nh || xh < 2  || yh < 2
%             keyboard
%         end
%         fh = (LSM_f_cubic_interpolation(xh,yh,h)/255-S(2))/S(1);
%
%         % enforce range [0,1]
%         fh=min(fh,1);
%         fh=max(fh,0);
%
%         wh = 1/vh(round(fh*255)+1)*S(1)^2;
%         fhi(i+Nf+1,j+Nf+1) = fh;
%
%         % take weighted average:
%         fe(i+Nf+1,j+Nf+1) = (wg*fg+wh*fh)/(wg+wh);
%     end
% end

fgi = fg_v;
fhi = fh_v;
fe = fe_v;

if sigma > 0
    % smooth: to be used for derivatives.
    fes = gaussFFT(fe,sigma,'G');
else
    fes = fe;
end

if pt > 1
    figure(13)
    subplot(1,2,1)
    hist(fgi(:)-fe(:),2*Nf+1);
    title(['mean(g-f) = ',num2str(mean(fgi(:)-fe(:)))])
    subplot(1,2,2)
    hist(fhi(:)-fe(:),2*Nf+1);
    title(['mean(h-f) = ',num2str(mean(fhi(:)-fe(:)))])
    Cov_fg=cov(fgi(:),fhi(:));
    title('Histogram of residuals of restaured images')
    %S1_est=sqrt(tan(0.5*atan2(2*Cov_fg(1,2),Cov_fg(2,2)-Cov_fg(1,1))))-1
end
end

