%% LSM 62 symmetric: estimate parameters
%
% function [est_x,d_x,est_sigma_0,NN,Red,Xm,v]  = ...
%     LSM_62_sym_warp_estimate_parameters(fe,fes,g,h,yi,zi,w,xa,pt)
%
% fe          = MfxMf estimated image
% fes         = MfxMf smoothed image
% g,h         = MgxMg, MhxMh observations of image
% yi,zi       = Kg, Kh list of observed pixels
% w           = Kg+Kh vector of weights
% xa          = actual estimates for geometric and radiometric transformation
% pt          boolean for plot/print
%
% est_x       = updated estimate (.x = 8-vector, .C = 8x8 CovM)
% d_x         = last correction of est_x.x
% est_sigma_0 = sqrt of sqrt of estimated variance factor
% NN          = number of observing pixels
% Red         = redundancy
% Xm          = design matrix
% v           = residuals
%
% wf 11/2018, 1/2020

function [est_x,d_x,est_sigma_0,NN,Red,Xm,v]  = ...
    LSM_62_sym_warp_estimate_parameters(fe,fes,g,h,yi,zi,w,xa,pt)

%% affinities
B    = [xa(1) xa(3) xa(5);xa(2) xa(4) xa(6);0 0 1];
Bi   = inv(B);
S    = [xa(7);xa(8)];
fak  = S(1);
faki = 1/fak;

%% half size of images
Nf = (size(fe,1)-1)/2;
Mg = size(g,1);
Mh = size(h,1);
Ng = (Mg-1)/2;
Nh = (Mh-1)/2;

%% number of observations
Kg   = size(yi,2);
Kh   = size(zi,2);

%% keynumbers of estimation
NN  = Kg+Kh;
NU  = 8;
% N - (8 geometric parameters + interior of fe)
% interior = round(sqrt(Kg*Kh))
Red = NN-(NU+round(sqrt(Kg*Kh)));

%
dg = zeros(NN,1);
Xm = zeros(NN,8);
%% Prepare derivatives of f
Derx = [3 10 3; 0 0 0; -3 -10 -3]/ 32; % Scharr's filter
fesx = conv2(fes,Derx ,'same');
fesy = conv2(fes,Derx','same');
% concatenate for simultaneous warping
fsxy(:,:,1)=fe';
fsxy(:,:,2)=fesx';
fsxy(:,:,3)=fesy';

%% first image
% backward  complete transform
BC = [1 0 Nf+1;0 1 Nf+1;0 0 1] * B * [1 0 -Ng-1;0 1 -Ng-1;0 0 1];
% forward complete transform (for maketform.m)
BCi  = inv(BC);
% provide transformation (transposed inverse)
TBC = maketform('affine',BCi');
% specify target window = g
xdata  = [1 size(g,1)];  ydata = [1 size(g,2)];
% simultanous warping of f, fx, fy
fsxygo  = imtransform(fsxy,TBC,'bicubic','xdata',xdata,'ydata',ydata);

% enforce same size if smaller than g
if size(fsxygo,1) < size(g,1)
    % size of fsxygo
    Mfo = size(fsxygo,1);
    Nfo = (Mfo-1)/2;
    % possible shift
    shift = max(0,Ng-Nfo);
    T = [1 0 0; 0 1 0; shift shift 1];
    TT = maketform('affine',T);
    fsxyg = imtransform(fsxygo,TT,'XData',xdata,'YData',ydata);
else
    fsxyg = fsxygo;
end

% extract
feh   = fsxyg(:,:,1)';
fesxg = fsxyg(:,:,2)';
fesyg = fsxyg(:,:,3)';
% estimated signal
feij_v  = feh(yi(1,:)'+Mg*(yi(2,:)'-1));
% fx of estimated signal
fxij_v  = fesxg(yi(1,:)'+Mg*(yi(2,:)'-1));
% fy of estimated signal
fyij_v  = fesyg(yi(1,:)'+Mg*(yi(2,:)'-1));
% residuals
dg_v    = g(yi(1,:)'+Mg*(yi(2,:)'-1))/255 - (feij_v-S(2))/S(1);
% transformed coordinates (Kg*3 matrix)
ij_v    = [yi(1,:)',yi(2,:)',ones(Kg,1)] * ([1 0 -Ng-1;0 1 -Ng-1;0 0 1])';
% transformed coordinates (Kg*3 matrix)
xc_v    = [yi(1,:)',yi(2,:)',ones(Kg,1)] * (B * [1 0 -Ng-1;0 1 -Ng-1;0 0 1])';
% Design matrix for additive correction of affinity
Xm_v    = [ fxij_v.*ij_v(:,1)   fyij_v.*ij_v(:,1) ...
    fxij_v.*ij_v(:,2)   fyij_v.*ij_v(:,2) ...
    fxij_v              fyij_v ...
    -(feij_v-S(2))*faki -ones(Kg,1)]*faki;

% % % sequential (old code)
% profile=zeros(Kg,5);
% for k = 1:Kg
%     i=yi(1,k)-Ng-1;                 % coordinates in g (centred)
%     j=yi(2,k)-Ng-1;
%     xh = B*[i;j;1];                 % real coordinates in f (centred)
%     xc=xh(1);
%     yc=xh(2);
%     x=xc+Nf+1;                      % pixel coordinates in f
%     y=yc+Nf+1;
%     fij  = LSM_f_cubic_interpolation(x,y,fe);    % estimated signal
%     fxij = LSM_fx_cubic_interpolation(x,y,fes);  % gradient at smoothed signal
%     fyij = LSM_fy_cubic_interpolation(x,y,fes);
%
%     dg(k)  = g(i+Ng+1,j+Ng+1)/255 - (fij-S(2))/S(1);    % g: 255->1
%     % Jacobian (see report)
%     Xm(k,:) = [fxij*i fyij*i fxij*j fyij*j fxij fyij -(fij-S(2))*faki -1]*faki;
%
%
%     profile(k,:)=[xc,yc,fij,fxij,fyij];
%     %     w(k) = 1/((1/w(k) - 1/12)*...
%     %         factors_V_cubic_interpolation(x)*...
%     %         factors_V_cubic_interpolation(y) +1/12);
% %     % Check Jacobian numerically, are ok
% %     % [my,Syy,J]=var_prop_classical(@f,mx,Sxx,p)
% %         if k < 10
% %             [fij_c,~,Jg] = var_prop_classical(@gj,xa,0.0000000001*ones(8),[i;j;fe(:)])
% %             (fij-S(2))/S(1)
% %             Xm(k,:)
% %             [fij_c-(fij-S(2))/S(1),Jg-Xm(k,:)]
% %         end
% end


%% second image
% backward  complete transform
BChi = [1 0 Nf+1;0 1 Nf+1;0 0 1] * Bi * [1 0 -Nh-1;0 1 -Nh-1;0 0 1];
% forward complete transform (for maketform.m)
BCh  = inv(BChi);
% provide transformation (transposed)
TBCh = maketform('affine',BCh');
% specify target window = h
xdata  = [1 size(h,1)];  ydata = [1 size(h,2)];
% simultanous warping of f, fx, fy
fsxyho  = imtransform(fsxy,TBCh,'bicubic','xdata',xdata,'ydata',ydata);

% enforce same size
if size(fsxyho,1) < size(h,1)
    % size of fsxyho
    Mfo = size(fsxyho,1);
    Nfo = (Mfo-1)/2;
    % possible shift
    shift = max(0,Nh-Nfo);
    T = [1 0 0; 0 1 0; shift shift 1];
    TT = maketform('affine',T);
    fsxyh = imtransform(fsxyho,TT,'XData',xdata,'YData',ydata);
else
    fsxyh = fsxyho;
end

% extract the three arrays f,fx,fy
feh   = fsxyh(:,:,1)';
fesxh = fsxyh(:,:,2)';
fesyh = fsxyh(:,:,3)';

% estimated signal
feij_v  = feh(zi(1,:)'+Mh*(zi(2,:)'-1));
% fx of estimated signal
fxij_v  = fesxh(zi(1,:)'+Mh*(zi(2,:)'-1));
% fy of estimated signal
fyij_v  = fesyh(zi(1,:)'+Mh*(zi(2,:)'-1));
% residuals
dg_v    = [dg_v;...
    h(zi(1,:)'+Mh*(zi(2,:)'-1))/255 - (S(1)*feij_v+S(2))];
% transformed coordinates (Kh*3 matrix)
xc_v    = [zi(1,:)',zi(2,:)',ones(Kh,1)] * (Bi * [1 0 -Nh-1;0 1 -Nh-1;0 0 1])';
% transform gradients
pij_v   = [fxij_v,fyij_v]*Bi(1:2,1:2);
pxij_v  = pij_v(:,1);
pyij_v  = pij_v(:,2);
% Design matrix
Xm_v    = [Xm_v;
    -[pxij_v.*xc_v(:,1) pyij_v.*xc_v(:,1) ...
    pxij_v.*xc_v(:,2) pyij_v.*xc_v(:,2) ...
    pxij_v            pyij_v ...
    -feij_v*faki      -ones(Kh,1)*faki]*fak];


% sequential (old code)
% profileh=zeros(Kh,7);
% fijh = zeros(Mh);
% for k=1:Kh;
%     i=zi(1,k)-Nh-1;                   % pixel coordinates in h (centred)
%     j=zi(2,k)-Nh-1;
%     xh = Bi*[i;j;1];                % real coordinates in f (centred)
%     xc=xh(1);
%     yc=xh(2);
%     x=xc+Nf+1;                      % pixel coordinates in f
%     y=yc+Nf+1;
%
%     fij  = LSM_f_cubic_interpolation(x,y,fe);    % estimated signal
%     fxij = LSM_fx_cubic_smoothing(x,y,fes);  % gradient at smoothed signal
%     fyij = LSM_fy_cubic_smoothing(x,y,fes);
%
%     pij  = Bi(1:2,1:2)'*[fxij;fyij]; % gradient in f
%     pxij = pij(1);
%     pyij = pij(2);
%     % residuals
%     dg(Kg+k)   = h(i+Nh+1,j+Nh+1)/255 - (S(1)*fij+S(2));     % h:255->1
%     % Jacobians  (see report)
%     Xm(Kg+k,:) = -[pxij*xc pyij*xc pxij*yc pyij*yc pxij pyij -fij*faki -faki]*fak;
%
%     profileh(k,:)=[xc,yc,fij,fxij,fyij,pxij,pyij];
%     fijh(zi(1,k),zi(2,k))= dg(Kg+k);
%
%
%     % Check Jacobian numerically, are ok
%     % [my,Syy,J]=var_prop_classical(@f,mx,Sxx,p)
%     %     if k < 10
%     %         [fij_c,~,Jh] = var_prop_classical(@hk,xa,0.0000000001*ones(8),[i;j;fe(:)])
%     %         (S(1)*fij+S(2))
%     %         Xm(Kg+k,:)
%     %         [fij_c-(S(1)*fij+S(2)),Jh-Xm(Kg+k,:)]
%     %     end
% end

%% take parallel results
Xm = Xm_v;
dg = dg_v;

%% Normal equation system
N  = Xm' * (Xm   .* repmat(w,1,NU));
n  = Xm' * (dg .* w);

%% Inverse and solution
C_xx     = inv(N);
d_x      = C_xx * n;

est_x.x  = xa + d_x;
est_x.C  = C_xx / 255^2;       % since weights refer to [0..255]

%% residuals
v     = Xm * d_x - dg;
Omega = v' * (v.*w) * 255^2;   % since weights refer to [0..255]
est_sigma_0 = sqrt(Omega/Red);


if pt > 1
    N           = N;
    n           = n';
    dx          = d_x'
    est_x_      = est_x.x'
    sigma_0_est = est_sigma_0
    covariance_2_sigma_rho(est_x.C);
    sigmas      = diag(sqrt(est_x.C))';
    B_est       = reshape(est_x.x(1:6),2,3)
    Sigmas_B    = reshape(sigmas(1:6),2,3)
    S_est       = est_x.x(7:8)'
    Sigmas_S    = sigmas(7:8)
    
    figure(13)
    subplot(1,2,1)
    hist(v(1:Kg),ceil(sqrt(Kg)))
    title({['v\_g: mean = ',num2str(mean(v(1:Kg)))],...
        ['v\_g: std  = ',num2str(std(v(1:Kg)))]})
    subplot(1,2,2)
    hist(v(Kg+1:Kg+Kh),ceil(sqrt(Kh)))
    title({['v\_h: mean = ',num2str(mean(v(Kg+1:Kg+Kh)))],...
        ['v\_h: std  = ',num2str(std(v(Kg+1:Kg+Kh)))]})
    
end

return