%% LSM_find_observation_positions in the common area
% with distance to boundary <= BOUND
%
% call:
% [Nf,yi,zi,w] =
% LSM_62_sym_warp_find_observation_positions(g,h,vg,vh,xa,pt)
%
% g, h   = two images
% vg,vh  = 256-vectors with variances of g and h: g, h in [0:1:255]
% xa     = parameters of approximate transformations B and S
%          xa(1:6) geoemtric parameters, xa(7:8) radiometric parameters
% Nf_MIN = minimum size of overlap (radius of window f)
% pt     = boolean for plot/print
%
% Nf     = size of refence image (radius/half side length)
% yi     = Kg vector, list of (i,j) in g, left upper corner [1,1]
% zi     = Kh-vector, list of (i,j) in h, left upper corner [1,1]
% w      = (Kg+Kh)-vectorvector of weights
%
% wf 08/2018

function [Nf,yi,zi,w]= ...
    LSM_62_sym_warp_find_observation_positions(g,h,vg,vh,xa,Nf_MIN,pt)
%% Constant, control
BOUND  = 3;

%% size of images
Mg  = size(g,1);
Mh  = size(h,1);
Ng=(Mg-1)/2;
Nh=(Mh-1)/2;

%% determine transfromation y->x, x->h
B  = [xa(1) xa(3) xa(5);xa(2) xa(4) xa(6);0 0 1];
Bi = inv(B);

%% find region in x-space
% transform boundary, take 3-pixel distance
Ngr   = Ng-BOUND;
bbg   = square_corners(Ngr);    % bb in g
bbgf  = B*bbg;                  % corresponding quadrangle in f
bbgfp = [bbgf,bbgf(:,1)];       % closed

Nhr   = Nh-BOUND;
bbh   = square_corners(Nhr);    % bb in h
bbhf  = Bi*bbh;                 % corresponding quadrangle in f
bbhfp = [bbhf,bbhf(:,1)];       % closed

%% find size of square, check 4 diagonals
% - determine intersection f of four diagonals l with four sides m of quadrangle
%   f=l x m, l = s x t, m = [0,0,1] x w, s = [a,b,1], t=[c,d,1], w=[u,v,0]
%   fx = (u*(a*d - b*c))/den, den = b*u - a*v + c*v - d*u
%   fy = (v*(a*d - b*c))/den
% - find minimum distance
dmin=10000;
% for each diagonal
for i=1:4
    % direction of i-th diagonal
    u   = cos(pi/4*(2*i-1));
    v   = sin(pi/4*(2*i-1));
    % for each side
    for j=1:4
        % distances to polygon g and h
        distg = dist_poly_ray(bbgfp,j,u,v);
        disth = dist_poly_ray(bbhfp,j,u,v);
        % take minimum
        dmin  = min([dmin,distg,disth]);
    end
end
% size of square of f
Nf  = floor((dmin)/sqrt(2)+0.0001);   %% +++++++++++++++++++++++++++++++
Nfr = Nf-BOUND;

if Nf < Nf_MIN
    yi=0;
    zi=0;
    w=0;
    return
end


%% pixels in the first image g
k=0;
yi=[];
fg=[];
w=[];

% parallel
[y,x] = meshgrid(1:Mg,1:Mg);
ijh = [x(:),y(:),ones(length(x(:)),1)];
xh_v = ijh * (B*[1 0 -Ngr-1; 0 1 -Ngr-1;0 0 1])';
indh = find((round(xh_v(:,1)) >= -Nfr) & (round(xh_v(:,1)) <= Nfr) ...
    & (round(xh_v(:,2)) >= -Nfr) & (round(xh_v(:,2)) <= Nfr));
%yi_v = [x(indh)'+BOUND;y(indh)'+BOUND];
yi_v = [x(indh)'+BOUND-1;y(indh)'+BOUND-1];
Kg = length(indh);
w_v = 1./vg(g(yi_v(1,:)'+Mg*(yi_v(2,:)'-1))+1);
w_v = w_v(:);

% % number of pixels in g
% Kg=size(yi,2);
yi=yi_v;

%% pixels in the second image z, away by bound from the boundary
k=0;
zi=[];
fh=[];
% parallel
[y,x] = meshgrid(1:Mh,1:Mh);
ijh = [x(:),y(:),ones(length(x(:)),1)];
xh_v = ijh * (Bi*[1 0 -Nhr-1; 0 1 -Nhr-1;0 0 1])';
indh = find((round(xh_v(:,1)) >= -Nfr) & (round(xh_v(:,1)) <= Nfr) ...
    & (round(xh_v(:,2)) >= -Nfr) & (round(xh_v(:,2)) <= Nfr));
zi_v = [x(indh)'+BOUND-1;y(indh)'+BOUND-1];
%zi_v = [x(indh)'+BOUND;y(indh)'+BOUND];
Kh = length(indh);
w_vh =1./vh(h(zi_v(1,:)'+Mh*(zi_v(2,:)'-1))+1);
w_vh = w_vh(:);
w_v = [w_v;w_vh];

% % number of pixels in h
% Kh=size(zi,2);

zi = zi_v;

% % enforce column of w
% w = w(:);

w= w_v;

if pt > 1
    
    figure(14)
    hold on
    axis equal
    Nfb=Nf-BOUND;
    % plot bb for f
    
    plot_quadrangle([-Nf,-Nf,Nf,Nf;...
        -Nf,Nf,+Nf,-Nf],'-r');
    % plot bb reduced for f
    plot_quadrangle([-Nfb,-Nfb,Nfb,Nfb;...
        -Nfb,Nfb,+Nfb,-Nfb],'--r')
    plot_quadrangle(bbgfp,'-b')
    plot_quadrangle(bbhfp,'-g')
    plot(1.5*[-Nf,Nf],1.5*[+Nf,-Nf],'--k')
    plot(1.5*[-Nf,Nf],1.5*[-Nf,+Nf],'--k')
    
    % second image  in coordinate system f
    xih = Bi*[zi-Nh*ones(2,Kh);ones(1,Kh)];
    plot(xih(2,:),xih(1,:),'og','MarkerSize',8)
    
    % first image in coordinate system f
    xig = B*[yi-Ng*ones(2,Kg);ones(1,Kg)];
    plot(xig(2,:),xig(1,:),'ob','MarkerSize',8)
    title('areas in middle image')
    
end
return