%% LS-matching 6+2 of two images g and h, symmetric
%
% model: observed images g and h referring to unknown reference f
%
% Basic model:
%
% g(y) -> (A,R) -> h(z)
%
% geometric model:    z   = z(y,A), A = A(p1, ..p6)
% radiometric model:  h   = h(g,R), R = R(p7,p8)
% unknown parameters: p   = p(1:8), to be estimated
%
%
%
% Relation to unknown signal f:
%
% g(y) -> (B,S) -> f(x) -> (B,S) -> h(z)
%
% geometric transformation:    B(a) = A^(-1/2)(u)
% radiometric transformation;  S(a) = R^(-1/2)(u)
% unknown parameters:          u = u(1:8), estimated internally
%
% affine geometric and radiometric transformations
% g(y) = a_7^-1 * (f(B    *   y + a) - a_8) + n_g(y)
% h(z) = a_7    * (f(B^-1 *  (z - a) + a_8) + n_h(z)
%
% B_a  = [a1 a3 a5;    S = [a7 a8
%         a2 a4 a6;          0  1]
%          0  0 1]
%
% all images are square:
%     size      (M x M), M odd
%     radius     N=(M-1)/2,
%     centre at (N+1,N+1)
%
% call:
% [est_x,est_sigma_0,NN,Mf,Red,N_iter] = ...
%    LSM_62_sym_warp_main(g,h,vg,vh,A_a,R_a,sigma_smooth,max_iter,pt)
%
% g            = Ng x Ng matrix of first image, type = double
% h            = Nh x Nh matrix of second image, type = double
% vg           = 256-vector of noise variances [0:1:255] of g
% vh           = 256-vector of noise variances [0:1:255] of h
% A_a          = 3x3 matrix, approximate transformation geometric parameters
%                   A_a  = [a b c;
%                           d e f;
%                           0 0 1]
%                   zh = A_a * yh
% R_a          = 1x2 vector of approximate contrast and intensity deviation
%                   h = R_a(1) * g + R_a(2)
% sigma_smooth = smoothing parameter for true image at first iteration
% max_iter     = maximum number of iterations
% Nf_MIN       = minimum size of overlap (radius of window f)
% pt           in (0,1,2) plot output control
%
% est_x       = estimated transformation parameters p with CovM
%               est_x.x  = parameters p of A and R
%               est_x.C  = covariance matrix
%               est_x.Ah = 3x3 homogeneous geometric affinity
%               est_x.R  = 1x2 radiometric parameters
%               1..6: geometric parameters
%               7..8: intenisty parameters
% est_sigma_0   estimated factor sigma_0 (should be close to 1)
% NN            number of observing pixels
% Mf            size of estimated image f
% Red           redundancy
% N_iter        number of iterations needed
% max_ratio     maximum ratio of corrections of parameters/sigma
%
% wf 7/2020

function [est_x,est_sigma_0,NN,Mf,Red,N_iter,max_ratio] = ...
    LSM_62_sym_warp_main(g,h,vg,vh,A_a,R_a,sigma_smooth,max_iter,Nf_MIN,pt)

%% ensure g and h are double
g=double(g);
h=double(h);

% check for postiveness
if det(A_a) < 0
    disp('Geometric affinity is not positive definite')
    est_x = 0;
    est_sigma_0 = 0;
    Red = -1;
    NN = 0;
    Mf = 0;
    N_iter = 0;
    return
end

%% test CPU-time
NO_TEST_CPU = 1;
sum_cpu=0;

%% determine approximate values: take squareroot of given affinities

% take squareroot from given affinities
B_a = sqrtm(A_a);
a7  = sqrt(R_a(1));
a8  = R_a(2)/(1+a7);
S_a = [a7,a8]';

% set approximate values
Be    = B_a(1:2,:);
xa    = [Be(:);S_a];

if pt > 0
    [im_lr,weite,border] = fill_left_right_images_0(g,h,max_iter);  
end

if pt > 1
    fig_conv = show_main_convergence_0(xa);
end

%% start iterations =============================================
est_sigma_0 = 1;
max_ratio   = 10;

for iter=1:max_iter
    
    if pt > 0
        disp(['Iteration No.: ',num2str(iter), ' -----------------------------'])
    end
    %% find observation positions
    if pt > 1
        tic;
    end
    if max_ratio > 0.5
        [Nf,yi,zi,w] = LSM_62_sym_warp_find_observation_positions(...
            g,h,vg,vh,xa,Nf_MIN,NO_TEST_CPU*pt);
    end
    if Nf < 5
        disp('Overlap is too small')
        est_x = 0;
        est_sigma_0 = 0;
        Red = -1;
        NN = 0;
        Mf = Nf;
        N_iter = iter;
        max_ratio = 100;
        return
    end
    
    %% determine estimated fe and smoothed fes
    
    [fe,fes,fg,fh] = LSM_62_sym_warp_estimate_f(...
        g,h,vg,vh,Nf,xa,sigma_smooth,NO_TEST_CPU*pt);
    if pt > 1
        cpu=toc;
        sum_cpu=sum_cpu+cpu;
    end
    % adapt smoothing parameter
    sigma_smooth=sigma_smooth*0.5;
    
    if pt > 0
        im_lr = fill_left_right_images(...
            im_lr,fg,fh,weite,iter);
    end
    
    %% estimate parameters
    old_est_sigma_0=est_sigma_0;
    if pt > 1
        old_cpu=cpu;
        tic;
    end
    
    % --------------  main routine  -------------------
    [est_xa,d_xa,est_sigma_0,NN,Red,Xm,dg]  = ...
        LSM_62_sym_warp_estimate_parameters(...
        fe,fes,g,h,yi,zi,w,xa,NO_TEST_CPU*pt);
    % --------------  main routine  -------------------
    
    if pt > 1
        cpu=cpu+toc;
        old_cpu=cpu;
        sum_cpu=sum_cpu+cpu;
    end
    
    if pt > 1
        show_main_convergence(...
            fig_conv,xa,est_xa,old_est_sigma_0,est_sigma_0,iter,old_cpu,cpu,sum_cpu);
    end
    xa = est_xa.x;
    
    %
    %% close iteration
    
    if pt > 1
        check=(Xm'*diag(w)*dg);
        check_AtWv=check'
    end
    max_ratio = max(abs(d_xa./sqrt(diag(est_xa.C))));
    %% plot/print
    if pt > 0
        display(['N = ',num2str(NN),', sigma_0 = ', num2str(est_sigma_0),', ratio = ',num2str(max_ratio)])
    end
   
    if  max_ratio < 0.1
        break
    end
    
end
N_iter = iter;
Mf = size(fe,1);


if pt > 0
    figure('name','undistorted images')
    imshow(im_lr)
    
    for iter = 1:max_iter
        text(border+(iter+0.5)*weite,1.75*border+weite,num2str(iter))
    end
end
%% Finale
est_x = finalize_sym_lsm(...
    iter,...
    xa,...
    est_xa,...
    pt);

return
