function filter = getFilter(sigma, kernel)
%  getFilter  returns the filter used by gaussFFT.
%
%  Input:
%     sigma     = kernel parameter 1x1
%     kernel    = kernel identifier, possible names are:
%                     'G',
%                     'Gx', 'Gy',
%                     'Gxx', 'Gyy', 'Gxy',
%                     'xG', 'yG',
%                     'x2G', 'y2G', 'xyG'
%
%  Output:
%     filter    = kernel in image space of size (2*3*sigma)^2

% Example:
%     surf(getFilter(3, 'Gx'));

% See also fspecial, diff, gaussFFT
% Copyright 2009-2011 Falko Schindler (mail@falkoschindler.de)

%% construct kernels
kSize = floor(4 * sigma);
G = conv2(fspecial('gaussian', [2 * kSize + 1, 1], sigma), ...
          fspecial('gaussian', [1, 2 * kSize + 1], sigma));
[y, x] = ndgrid(-kSize : kSize);
dy = [-0.5, 0, 0.5]';
dx = [-0.5, 0, 0.5];
dyy = [0.25, -0.5, 0.25]';
dxx = [0.25, -0.5, 0.25];
dxy = conv2(dx, dy);
switch kernel
    case 'G',   filter = G;
    case 'Gy',  filter = imfilter(G, dy, 'same', 'corr');
    case 'Gx',  filter = imfilter(G, dx, 'same', 'corr');
    case 'Gyy', filter = imfilter(G, dyy, 'same', 'corr');
    case 'Gxx', filter = imfilter(G, dxx, 'same', 'corr');
    case 'Gxy', filter = imfilter(G, dxy, 'same', 'corr');
    case 'yG',  filter = y .* G;
    case 'xG',  filter = x .* G;
    case 'y2G', filter = y.^2 .* G;
    case 'x2G', filter = x.^2 .* G;
    case 'xyG', filter = x .* y .* G;
    otherwise,  error 'gaussFFT: unknown kernel name';
end
