function [s,means,sigma] = noise_standard_deviation_estimation(img,K)
% Estimate noise variance function for each channel of image IMG.
%
% img     = image with c channels
% K       = number of variances (default = 10)
%
% s       = c x 256 [0:1:255] lookup table for noise standard deviation
% means   = Kx1 vector of mean intensities, where variance is stimated
% sigmas  = Kx1 ve ctor of estimated variances
%
% For each variance to be estimated we select corresponding image pixels
% and robustly estimate their variance from their mean gradient magnitude.
% We iteratively exclude pixels with large gradients that are probably
% caused by image content rather than noise.
% See W. Foerstner, "Image Preprocessing for Feature Extraction in Digital
% Intensity, Color and Range Images" (2000), Eq. (32) for more details.
%
% For normalizing the image we need to assign a noise variance to each
% discrete gray value. Therefore we perform a spline interpolation of the
% discrete samples from the previous step.
%
% Taken from noiseNormalize.m
% 
% See also: sfop
%
% Licence:
%   For internal use only.
%
% Authors:
%   Timo Dickscheid, Falko Schindler, Wolfgang Foerstner
%   Department of Photogrammetry
%   Institute of Geodesy and Geoinformation
%   University of Bonn
%   Bonn, Germany
%
% Contact person:
%   Falko Schindler (falko.schindler@uni-bonn.de)
%
% Copyright 2012-2012

% TODO: How to define number of bins B and discretization (here 256)?
% up to now: input or default = 10
%% default values
K(nargin < 2)       = 10;


%% handle multiple channels
if size(img, 3) > 1
    means = zeros(size(img, 3), K);
    sigma = zeros(size(img, 3), K);
    for c = 1 : size(img, 3)
        [s(c,:),means(c,:),sigma(c,:)] = noise_standard_deviation_estimation(img(:, :, c),K);
    end
    return
else
    
end
%% check type of image
if max(img(:)) <= 255
    % range[0,255]
    type_img = 255;
    % scale image
    img=double(img)/255;
end
if max(img(:)) <= 1
    % range [0,1]
    type_img = 1;
end

%% Image gradients
% We will estimate variances from image gradients, thus need to convolve
% the image with two derivative kernels and add the squared gradients.
grad = imfilter(img, [-1, 0, 1], 'replicate').^2 + ...
       imfilter(img, [-1; 0; 1], 'replicate').^2;

%% Variance estimation
% First we sort all pixels and their gradients with ascending gray value.

[F, idx] = sort(img(:));

% collect gradients
G = grad(idx);
%%
% We initialize storage for a certain number of variance estimates with
% their corresponding mean gray value.
[means, sigma] = deal(zeros(1, K));
%% 
% For each variance to be estimated we select corresponding image pixels
% and robustly estimate their variance from their mean gradient magnitude.
% We iteratively exclude pixels with large gradients that are probably
% caused by image content rather than noise.
% See W. Foerstner, "Image Preprocessing for Feature Extraction in Digital
% Intensity, Color and Range Images" (2000), Eq. (32) for more details.
for b = 1 : numel(means)
    selection = round((b - 1) / numel(means) * numel(img) + 1) : ...
                round( b      / numel(means) * numel(img));
    g = G(selection);
    mu = mean(g);
    for iter = 1 : 3
        mu = 2.392211191177332 * mean(g(g < mu));
    end
    sigma(b) = sqrt(mu / 4);
    means(b) = mean(F(selection));
end
% regularize means, if image is under- or overexposed
% prior
means0=([1:K]-0.5)/K; w = 0.01;
% weighted mean
means = w*means0 +(1-w)*means;
%% Interpolation
% For normalizing the image we need to assign a noise variance to each
% discrete gray value. Therefore we perform a spline interpolation of the
% discrete samples from the previous step
g_domain = linspace(0, 1, 256);
if K > 1 
    s = interp1(means, sigma, g_domain, 'linear');
end;
% and set the values outside the given range to the first and last value
s(g_domain < means(1)) = sigma(1);
s(g_domain > means(end)) = sigma(end);
% limit s from below by rounding error
s=max(sqrt(1/12)/255,s);
% scale s
if type_img == 255
    s=s*255;
end
return